﻿using System.Collections.Generic;
using System.IO;
using System.Xml.Serialization;
using System.Windows.Forms;

namespace TVSharp
{
    public class SettingsPersister
    {
        private const string FreqMgrFilename = "settings.xml";

        private readonly string _settingsFolder;

        public SettingsPersister()
        {
            _settingsFolder = Path.GetDirectoryName(Application.ExecutablePath);
        }

        public SettingsMemoryEntry ReadSettings()
        {
            var result = ReadObject<SettingsMemoryEntry>(FreqMgrFilename);
            if (result != null)
            {
                return result;
            }
            return new SettingsMemoryEntry();
        }

        public void PersistSettings(SettingsMemoryEntry entries)
        {
            WriteObject(entries, FreqMgrFilename);
        }

        private T ReadObject<T>(string fileName)
        {
            var filePath = Path.Combine(_settingsFolder, fileName);
            if (File.Exists(filePath))
            {
                using (var fileStream = new FileStream(filePath, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite))
                {
                    var ser = new XmlSerializer(typeof(T));
                    return (T)ser.Deserialize(fileStream);
                }
            }
            return default(T);
        }

        private void WriteObject<T>(T obj, string fileName)
        {
            var filePath = Path.Combine(_settingsFolder, fileName);
            using (FileStream fileStream = new FileStream(filePath, FileMode.Create))
            {
                var ser = new XmlSerializer(obj.GetType());
                ser.Serialize(fileStream, obj);
            }
        }
    }
}
